# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.2.0

import enum

import winrt.system
from winrt._winrt_windows_devices_i2c import (
    I2cTransferResult,
    I2cConnectionSettings,
    I2cController,
    I2cDevice,
    _II2cDeviceStatics,
    II2cDeviceStatics,
)

__all__ = [
    "I2cBusSpeed",
    "I2cSharingMode",
    "I2cTransferStatus",
    "I2cTransferResult",
    "I2cConnectionSettings",
    "I2cController",
    "I2cDevice",
    "II2cDeviceStatics",
]

class I2cBusSpeed(enum.IntEnum):
    STANDARD_MODE = 0
    FAST_MODE = 1

class I2cSharingMode(enum.IntEnum):
    EXCLUSIVE = 0
    SHARED = 1

class I2cTransferStatus(enum.IntEnum):
    FULL_TRANSFER = 0
    PARTIAL_TRANSFER = 1
    SLAVE_ADDRESS_NOT_ACKNOWLEDGED = 2
    CLOCK_STRETCH_TIMEOUT = 3
    UNKNOWN_ERROR = 4

